package org.hisp.dhis.mobile.reporting.model;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Vector;

public class Activity implements ISerializable {

	private Beneficiary beneficiary;

	private Task task;

	private Date dueDate;

	private boolean late;
	
	public Activity() {
	}

	// Getter and Setter
	
	
	
	public Beneficiary getBeneficiary() {
		return beneficiary;
	}

	public boolean isLate() {
		return late;
	}

	public void setLate(boolean late) {
		this.late = late;
	}

	public void setBeneficiary(Beneficiary beneficiary) {
		this.beneficiary = beneficiary;
	}

	public Task getTask() {
		return task;
	}

	public void setTask(Task task) {
		this.task = task;
	}

	public Date getDueDate() {
		return dueDate;
	}

	public void setDueDate(Date dueDate) {
		this.dueDate = dueDate;
	}

	public static Activity recordToActivity(byte[] rec) {
		ByteArrayInputStream bin = new ByteArrayInputStream(rec);
		DataInputStream din = new DataInputStream(bin);

		Activity activity = new Activity();
		Beneficiary beneficiary = new Beneficiary();
		Task task = new Task();
		try {
		        // Read Beneficiary Information
			beneficiary.setId(din.readInt());
			beneficiary.setLastName(din.readUTF());
			beneficiary.setMiddleName(din.readUTF());
			beneficiary.setFirstName(din.readUTF());
			activity.setLate(din.readBoolean());
			Vector atts = new Vector();
			beneficiary.setAttsValues( atts );
			int numAtt = din.readInt();
                        for(int i = 0; i < numAtt; i++){
                            atts.addElement(din.readUTF());                         
                        }
                        
                        // Write Due Date
			activity.setDueDate(new Date(din.readLong()));
			
			// Write Task Information
			task.setProgStageInstId(din.readInt());
			task.setProgStageId(din.readInt());
			task.setComplete(din.readBoolean());

			activity.setTask(task);
			activity.setBeneficiary(beneficiary);
		} catch (IOException ioe) {
		}

		return activity;
	}

	public static byte[] activityToRecord(Activity activity) {
		ByteArrayOutputStream deOs = new ByteArrayOutputStream();
		DataOutputStream dout = new DataOutputStream(deOs);

		try {
		        // Write Beneficiary Information
			dout.writeInt(activity.getBeneficiary().getId());
			dout.writeUTF(activity.getBeneficiary().getLastName());
			dout.writeUTF(activity.getBeneficiary().getMiddleName());
			dout.writeUTF(activity.getBeneficiary().getFirstName());
			dout.writeBoolean(activity.isLate());
			Vector atts = activity.getBeneficiary().getAttsValues();
                        int numAtt = atts.size();
                        dout.writeInt(numAtt);
                        for(int i = 0; i < numAtt; i++){
                            dout.writeUTF( (String)atts.elementAt( i ) );
                        }
                        
			// Write Due Date
			dout.writeLong(activity.getDueDate().getTime());
			
			// Write Task Information
			dout.writeInt(activity.getTask().getProgStageInstId());
			dout.writeInt(activity.getTask().getProgStageId());
			dout.writeBoolean(activity.getTask().isComplete());
			dout.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return deOs.toByteArray();
	}

	public void deSerialize(DataInputStream din) throws IOException {

		this.setDueDate(new Date(din.readLong()));
		this.beneficiary.deSerialize(din);
		this.task.deSerialize(din);
	}

	public byte[] serialize() throws IOException {

		ByteArrayOutputStream deOs = new ByteArrayOutputStream();
		DataOutputStream dout = new DataOutputStream(deOs);

		try {
			// Write Beneficiary Information
			dout.writeInt(this.getBeneficiary().getId());
			dout.writeUTF(this.getBeneficiary().getLastName());
			dout.writeUTF(this.getBeneficiary().getMiddleName());
			dout.writeUTF(this.getBeneficiary().getFirstName());
			// Write Due Date
			dout.writeLong(this.getDueDate().getTime());
			// Write Task Information
			dout.writeInt(this.getTask().getProgStageInstId());
			dout.writeInt(this.getTask().getProgStageId());
			dout.writeBoolean(this.getTask().isComplete());
			dout.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return deOs.toByteArray();
	}

	public void deSerialize(byte[] data) throws IOException {
		// TODO Auto-generated method stub

	}
}
